@@ -39,6 +39,8 @@ def serialize_bookmark(marker):
         containing values of the ordering columns, and `backwards` denotes the
         paging direction.
     :returns: A CSV-like string using ``~`` as a separator."""
+    if marker is None:
+        return None
     x, backwards = marker
     ss = s.serialize_values(x)
     direction = "<" if backwards else ">"
@@ -102,9 +104,16 @@ class Page(list):
 
 
 class Paging:
-    """Object with paging information. Most properties return a page marker.
+    """Metadata describing the position of a page in a collection.
+    Most properties return a page marker.
     Prefix these properties with ``bookmark_`` to get the serialized version of
-    that page marker."""
+    that page marker.
+
+    Unless you're extending sqlakeyset you should not be constructing this
+    class directly - use sqlakeyset.get_page or sqlakeyset.select_page to
+    acquire a Page object, then access page.paging to get the paging
+    metadata.
+    """
 
     def __init__(
         self,
@@ -113,18 +122,17 @@ class Paging:
         ocols,
         backwards,
         current_marker,
-        get_marker=None,
         markers=None,
+        get_keys_from=None,  # used only in unit tests
     ):
-
         self.original_rows = rows
 
-        if get_marker:
+        if get_keys_from:
 
-            def _get_marker(i):
-                return get_marker(self.original_rows[i], ocols)
+            def _get_keys_at(i):
+                return get_keys_from(self.original_rows[i], ocols)
 
-            marker = _get_marker
+            marker = _get_keys_at
         else:
             if rows and not markers:
                 raise ValueError
@@ -132,6 +140,7 @@ class Paging:
 
         self.per_page = per_page
         self.backwards = backwards
+        self._get_keys_at = marker
 
         excess = rows[per_page:]
         rows = rows[:per_page]
@@ -230,11 +239,60 @@ class Paging:
         requested in ``per_page``."""
         return len(self.rows) == self.per_page
 
-    def __getattr__(self, name):
-        PREFIX = "bookmark_"
-        if name.startswith(PREFIX):
-            _, attname = name.split(PREFIX, 1)
-            x = getattr(self, attname)
-            return serialize_bookmark(x)
+    def get_marker_at(self, i):
+        """Get the marker for item at the given row index."""
+        return self._get_keys_at(i), self.backwards
 
-        raise AttributeError
+    def get_bookmark_at(self, i):
+        """Get the bookmark for item at the given row index."""
+        return serialize_bookmark(self.get_marker_at(i))
+
+    def items(self):
+        """Iterates over the items in the page, returning a tuple ``(marker,
+        item)`` for each."""
+        for i, row in enumerate(self.rows):
+            yield self.get_marker_at(i), row
+
+    def bookmark_items(self):
+        """Iterates over the items in the page, returning a tuple ``(bookmark,
+        item)`` for each."""
+        for i, row in enumerate(self.rows):
+            yield self.get_bookmark_at(i), row
+
+    # The remaining properties are just convenient shorthands to avoid manually
+    # calling serialize_bookmark.
+    @property
+    def bookmark_next(self):
+        """Bookmark for the next page (in the original query order)."""
+        return serialize_bookmark(self.next)
+
+    @property
+    def bookmark_previous(self):
+        """Bookmark for the previous page (in the original query order)."""
+        return serialize_bookmark(self.previous)
+
+    @property
+    def bookmark_current_forwards(self):
+        """Bookmark for the current page in forwards direction."""
+        return serialize_bookmark(self.current_forwards)
+
+    @property
+    def bookmark_current_backwards(self):
+        """Bookmark for the current page in backwards direction."""
+        return serialize_bookmark(self.current_backwards)
+
+    @property
+    def bookmark_current(self):
+        """Bookmark for the current page in the current paging direction."""
+        return serialize_bookmark(self.current)
+
+    @property
+    def bookmark_current_opposite(self):
+        """Bookmark for the current page in the opposite of the current
+        paging direction."""
+        return serialize_bookmark(self.current_opposite)
+
+    @property
+    def bookmark_further(self):
+        """Bookmark for the following page in the current paging direction."""
+        return serialize_bookmark(self.further)
